<?php
   /*
     * $Id$
     *
     * MAIA MAILGUARD LICENSE v.1.0
     *
     * Copyright 2010 by Robert LeBlanc <rjl@renaissoft.com>
     *                   David Morton <mortonda@dgrmm.net>
     * All rights reserved.
     *
     * PREAMBLE
     *
     * This License is designed for users of Maia Mailguard
     * ("the Software") who wish to support the Maia Mailguard project by
     * leaving "Maia Mailguard" branding information in the HTML output
     * of the pages generated by the Software, and providing links back
     * to the Maia Mailguard home page.  Users who wish to remove this
     * branding information should contact the copyright owner to obtain
     * a Rebranding License.
     *
     * DEFINITION OF TERMS
     *
     * The "Software" refers to Maia Mailguard, including all of the
     * associated PHP, Perl, and SQL scripts, documentation files, graphic
     * icons and logo images.
     *
     * GRANT OF LICENSE
     *
     * Redistribution and use in source and binary forms, with or without
     * modification, are permitted provided that the following conditions
     * are met:
     *
     * 1. Redistributions of source code must retain the above copyright
     *    notice, this list of conditions and the following disclaimer.
     *
     * 2. Redistributions in binary form must reproduce the above copyright
     *    notice, this list of conditions and the following disclaimer in the
     *    documentation and/or other materials provided with the distribution.
     *
     * 3. The end-user documentation included with the redistribution, if
     *    any, must include the following acknowledgment:
     *
     *    "This product includes software developed by Robert LeBlanc
     *    <rjl@renaissoft.com>."
     *
     *    Alternately, this acknowledgment may appear in the software itself,
     *    if and wherever such third-party acknowledgments normally appear.
     *
     * 4. At least one of the following branding conventions must be used:
     *
     *    a. The Maia Mailguard logo appears in the page-top banner of
     *       all HTML output pages in an unmodified form, and links
     *       directly to the Maia Mailguard home page; or
     *
     *    b. The "Powered by Maia Mailguard" graphic appears in the HTML
     *       output of all gateway pages that lead to this software,
     *       linking directly to the Maia Mailguard home page; or
     *
     *    c. A separate Rebranding License is obtained from the copyright
     *       owner, exempting the Licensee from 4(a) and 4(b), subject to
     *       the additional conditions laid out in that license document.
     *
     * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER AND CONTRIBUTORS
     * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
     * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
     * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
     * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
     * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
     * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
     * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
     * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
     * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
     * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
     *
     */


  function pre_check_10() {
    global $dbh;

    if (index_exists("maia_users", "primary_email_idx")) {
      return array(false, "primary_email_idx already exists");
    } else {
      return array(true,"");
    }
  }

  function upgrade_10() {
    global $dbh;

    $bad_key_query = <<<EOQ
      select maia_users.id
      from users left join maia_users on users.maia_user_id = maia_users.id
      where maia_user_id in
         (SELECT maia_users.id
           FROM maia_users left join users
           on maia_users.primary_email_id = users.id
           where maia_user_id != maia_users.id)
EOQ;
    $update = <<<EOQ
        update users left join maia_users on users.maia_user_id = maia_users.id
          set primary_email_id = users.id
          where maia_user_id = ?
EOQ;
    if (is_mysql()) {
      $alter = <<<EOQ
        ALTER TABLE maia_users ADD UNIQUE primary_email_idx ( primary_email_id )
EOQ;
    } else {
      $alter = <<<EOQ
        CREATE UNIQUE INDEX primary_email_idx ON maia_users (primary_email_id)
EOQ;
    }

    $sth = $dbh->query($bad_key_query);
    if (PEAR::isError($sth)) {
      $str = $sth->getMessage() . " = [" . $bad_key_query . "]" ;
      return array(false, $str );
    }
    while ($row=$sth->fetchrow()) {
      $result = $dbh->query($update, array($row['id']));
      if (PEAR::isError($result)) {
        $str = $result->getMessage() . " = [" . $update . "]" ;
        return array(false, $str );
      }
    }
    // now we should have gotten rid of any duplicate keys, update index
    $result = $dbh->query($alter);
    if (PEAR::isError($result)) {
      $str = $result->getMessage() . " = [" . $alter . "]" ;
      return array(false, $str );
    }

    return array(true,"");
  }

  function post_check_10() {
    global $dbh;

    if (! index_exists("maia_users", "primary_email_idx")) {
      return array(false, "Error creating index primary_email_idx");
    } else {
      return array(true,"");
    }

  }

?>
